/*
 * dht.c
 *
 * Created: 2014-09-15 09:13:52
 *  Author: dsosnowski
 */ 

//------- includes ---------------
#define F_CPU 9600000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "defs.h"
#include "dht.h"

#define HDDR			DDRB
#define HPORT			PORTB
#define HPIN			PINB
#define DHTPIN			4

#define DEF_TIMEOUT		1000
#define DHT_FRAME		41

/*
#define RESULT_READY	0
#define WORKING			1
#define ERROR			2
*/

struct {
	int16_t humdity;	
	int16_t temperature;
	uint16_t hdata;
	uint16_t tdata;
	char check;
	char status;
} dht;

//char datatable[44];

//-------------------------------------------------------
char DHTRead(void)
{
	int16_t timeout;
	char status;
	char i;
	char limit;
	
	//prepare
	dht.hdata = 0;
	dht.tdata = 0;
	dht.check = 0;
	dht.status = 0;
	
	HDDR &= ~(1<<DHTPIN);
	HPORT |= (1<<DHTPIN);
	status = 0;
	timeout = 0;
	while (status == 0)
	{
		if ((HPIN & (1<<DHTPIN)) == 0) timeout++;
		else status = 1;
		if (timeout > DEF_TIMEOUT) 
			status = DHT_ERR1;			//propably short to GND
	}
	
	//init
	if (status == 1)
	{
		HDDR |= (1<<DHTPIN);
		HPORT |= (1<<DHTPIN);
		_delay_ms(10);
		
		//start low pulse
		HPORT &= ~(1<<DHTPIN);
		_delay_ms(25);
		HPORT |= (1<<DHTPIN);
		HDDR &= ~(1<<DHTPIN);	// - go to Hi
		
		status = 0;
		timeout = 0;
		while (status == 0)
		{
			if ((HPIN & (1<<DHTPIN))) timeout++;
			else status = 1;
			if (timeout > DEF_TIMEOUT) 
				status = DHT_ERR2;		//propably no DHT
		}
	}
	
	//transmition
	if (status == 1)
	{
		i = 0;
		while (i < DHT_FRAME)
		{
			status = 0;
			timeout = 0;
		
			while (status == 0)
			{
				if ((HPIN & (1<<DHTPIN)) == 0) timeout++;
				else status = 1;
				if (timeout > DEF_TIMEOUT)
				{
					status = DHT_ERR3;		//propably short to gnd - transmit error
					i = DHT_FRAME;
				}
			}

			timeout = 0;
			while (status == 1)
			{
				if ((HPIN & (1<<DHTPIN))) timeout++;
				else status = 2;
				if (timeout > DEF_TIMEOUT)
				{
					status = DHT_ERR3;		//propably no DHT - transmit error
					i = DHT_FRAME;
				}
			}
			
			if (status == 2)
			{
				if (i == 0) limit = (timeout / 2);
				
				if (i <= 16)
				{
					dht.hdata <<= 1;
					if (timeout > limit) dht.hdata |= 1;
				}
				
				if (i <= 32)
				{
					dht.tdata <<= 1;
					if (timeout > limit) dht.tdata |= 1;
				}
				if (i <= 40)
				{
					dht.check <<= 1;
					if (timeout > limit) dht.check |= 1;
				}
			}
			i++;
		}
		if (i == DHT_FRAME) status = DHT_READY;
	}

	dht.status = status;
	return status;
}

//-------------------------------------------------------
char DHTStatusGet(void)
{
	char i;
	i = (dht.hdata >> 8);
	i += (dht.hdata & 0xFF);
	i += (dht.tdata >> 8);
	i += (dht.tdata & 0xFF);
	
	if (i == dht.check) return 1; 
	else return 0;
}

//-------------------------------------------------------
int16_t DHTHumidityGet(void)
{
	if (dht.status == DHT_READY)
	{
		dht.humdity = (dht.hdata >> 8);
		//dht.humdity *= 10;
		//dht.humdity += ((dht.hdata) & 0x0F);
	}
	else
	{
		dht.humdity = 0;
	}
	return dht.humdity;
}

//-------------------------------------------------------
int16_t DHTTemperatureGet(void)
{
	if (dht.status == DHT_READY)
	{
		dht.temperature = (dht.tdata >> 8);
		dht.temperature *= 10;
		dht.temperature += ((dht.tdata) & 0x0F);
	}
	else
	{
		dht.temperature = 0;
	}
	return dht.temperature;
}

