


#define F_CPU 12000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "s_uart.h"

#define CORE_CLOCK		9600000
#define BAUD_RATE		9600

#define TIMER_DIV		8
#define TIMER_CLOCK		(CORE_CLOCK/TIMER_DIV)
#define BAUD_FACTOR		(TIMER_CLOCK/BAUD_RATE)	//1 int on 1 bit

#define SUART_PORT		PORTB
#define SUART_PIN		PINB
#define SUART_DDR		DDRB
#define TXD 			0 //output

#define TXD_SET			SUART_PORT |= (1<<TXD)
#define TXD_CLR			SUART_PORT &= ~(1<<TXD)
	
#define START_TIMER		TCCR0B = (1<<WGM02 | 1<<CS01)
#define STOP_TIMER		TCCR0B = (1<<WGM02)
#define TIMER_VAL		TCNT0
#define CLEAR_TIMER		TCNT0 = 0


//#define SUART_BUFFER	32

#define LED 0 //H - on, L - off


//i.e        |         |          |
//"VV" = -----__--_-_-_-__--_-_-_------



//-------------------------------------------
struct {
	volatile char s_byte;
	volatile char s_index;
	char s_buffer[SUART_SEND_BUFFER];
	volatile char status;
} suart;

struct {		
	volatile char status;
	char index;
	char code;
	char adr;
	char val;
	char param;
	char ready;
	char invailid;
} command;

//-------------------------------------------
void SuartInit(void)
{
	 SUART_PORT |= (1<<TXD);		//config ports
     SUART_DDR |= (1<<TXD);
   
     TCCR0A = (1<<WGM01 | 1<<WGM00);
	 TCCR0B = (1<<WGM02);			//fast PWM
	 OCR0A = BAUD_FACTOR;			//S_uart freq  

	 TIFR0 = (0xFF);				//clear flag
 	 TIMSK0 = (1<<OCIE0A);			//enable int		
	
	 //START_TIMER;	 
	 sei();
}

//-------------------------------------------
void SuartSend(char data)
{
	//_delay_ms(2);

	CLEAR_TIMER;					//first - idle cycle
	START_TIMER;					//start

	//while(data)
	{
 		suart.status = 1;			//transmit mode
		suart.s_byte = 0;			//start bit first
		suart.s_index = 10;			//set cycles
		while(suart.s_index >= 10);	//wait for star bit
		suart.s_byte = data;		//data
		while(suart.s_index > 1);		//wait for complete and next
		suart.s_byte = 0xFF;			//stop_bit
		while(suart.s_index > 0);		//wait for complete and next
		//data++;						//next data
	}
	suart.status = 0;				//receive mode
	STOP_TIMER;
	//_delay_ms(2);
}

//-------------------------------------------
void SuartClear(void)
{
	suart.status = 0;				//receive mode
	command.status = 0;
}

//-------------------------------------------
SIGNAL(TIM0_COMPA_vect)
{	      
	static char r_byte = 0xFF;
	
	//---------------------------------------------
	if (suart.status == 1)		//if any byte to send
	{
		if (suart.s_byte & 1) TXD_SET; 
		else		   		  TXD_CLR;
		suart.s_byte>>=1;			//next bit
		suart.s_index--;			//decrement counter
	}
	
}
